# frozen_string_literal: true
module NexClient
  module Commands
    module CubeInstances
      extend Helpers

      CUBES_TITLE = "Cube Instances".colorize(:blue)
      CUBES_HEADERS = ['id','status','container','region','storage','ip','port','cluster'].map(&:upcase)

      SNAPSHOTS_TITLE = "Snapshots".colorize(:magenta)
      SNAPSHOTS_HEADERS = ['created_at','rack ip','path'].map(&:upcase)

      def self.list(args,opts)
        filters = {}
        filters[:status] = opts.status if opts.status.present?
        filters[:persistent_storage] = opts.storage if opts.storage.present?

        # All option
        filters = {} if opts.all

        # Identification options
        filters[:'app.name'] = opts.app if opts.app.present?
        filters[:'addon.name'] = opts.addon if opts.addon.present?

        # Create table
        list = NexClient::CubeInstance.includes(:cluster).where(filters).order('status')
        self.display_cubes(list)

        # Loop through results
        while (list.pages.links||{})['next']
          return true if ask("Press enter for next page ('q' to quit)") =~ /q/
          list = list.pages.next
          self.display_cubes(list)
        end
      end

      def self.info(args,opts)
        id = args.first
        e = NexClient::CubeInstance.includes(:cluster).find(uuid: id).first

        # Display cube information
        self.display_cubes(e)
      end

      # Retrieve application logs from all containers
      def self.logs(args,opts)
        name = args.first
        e = NexClient::CubeInstance.find(uuid: name).first

        # Display error
        unless e
          error("Error! Could not find cube: #{name}")
          return false
        end

        # Retrieve logs and display them
        logs = e.logs(tail: opts.tail).first
        self.display_logs(logs.log_ret)
      end

      def self.snapshots(args,opts)
        name = args.first
        e = NexClient::CubeInstance.find(uuid: name).first

        # Display error
        unless e
          error("Error! Could not find cube: #{name}")
          return false
        end

        # Retrieve snapshots and display them
        snapshots = e.snapshots.first['snapshots']
        self.display_snapshots(snapshots)
      end

      def self.trigger_action(action,args,opts)
        id = args.first
        e = NexClient::CubeInstance.find(uuid: id).first

        # Display error
        unless e
          error("Error! Could not find cube: #{id}")
          return false
        end

        # Perform
        e.send(action)

        # Display errors if any
        if e.errors.any?
          display_record_errors(e)
          return false
        end

        success("Initiated #{action} for cube #{id}...")
      end

      # SSH to the addon
      def self.ssh(args,opts)
        id = args.first
        e = NexClient::CubeInstance.find(uuid: id).first

        # Display error
        unless e
          error("Error! Could not find cube: #{id}")
          return false
        end

        # Perform command
        perform_ssh_cmd(e.ssh_cmd_template)
      end

      # Download a file from a container
      def self.download(args,opts)
        id = args.first
        remote_path = args[1]
        local_path = args[2] || './'
        e = NexClient::CubeInstance.includes(:compute_rack).find(uuid: id).first

        # Display error
        unless e
          error("Error! Could not find cube: #{id}")
          return false
        end

        # Get rack and file details
        rack = e.compute_rack
        filename = File.basename(remote_path)

        # Copy and download file
        with_cert_identity do |username,pv_key_path|
          cmds = []
          tmp_folder = "/home/#{username}/tmp"
          tmp_path = "#{tmp_folder}/#{filename}"
          ssh_cmd = format_cmd(rack.ssh_cmd_template, username: username, certfile: pv_key_path)

          # Format tmp folder creation command
          cmds << "#{ssh_cmd} mkdir -p #{tmp_folder}"

          # Format container copy command
          cmds << "#{ssh_cmd} sudo docker cp #{e.container}:#{remote_path} #{tmp_folder}/"

          # Format server copy command
          cmds << format_cmd(rack.scp_cmd_template,
            username: username,
            certfile: pv_key_path,
            remote_path: tmp_path,
            dst_local_file: local_path
          )

          # Cleanup
          cmds << "#{ssh_cmd} rm -f #{tmp_path}"

          # Execute commands
          cmds.each { |cmd| system(cmd) }
        end
      end

      # Upload a file to the container
      def self.upload(args,opts)
        id = args.first
        local_path = args[1]
        remote_path = args[2] || "/tmp/"
        e = NexClient::CubeInstance.includes(:compute_rack).find(uuid: id).first

        # Display error
        unless e
          error("Error! Could not find cube: #{id}")
          return false
        end

        # Get rack and file details
        rack = e.compute_rack
        filename = File.basename(local_path)

        # Upload and copy file
        with_cert_identity do |username,pv_key_path|
          cmds = []
          tmp_folder = "/home/#{username}/tmp"
          tmp_path = "#{tmp_folder}/#{filename}"
          ssh_cmd = format_cmd(rack.ssh_cmd_template, username: username, certfile: pv_key_path)

          # Format tmp folder creation command
          cmds << "#{ssh_cmd} mkdir -p #{tmp_folder}"

          # Format server copy command
          cmds << format_cmd(rack.scp_cmd_template,
            username: username,
            certfile: pv_key_path,
            src_local_file: local_path,
            remote_path: tmp_path
          )

          # Format container copy command
          cmds << "#{ssh_cmd} sudo docker cp #{tmp_path} #{e.container}:#{remote_path}"

          # Cleanup
          cmds << "#{ssh_cmd} rm -f #{tmp_path}"

          # Execute commands
          cmds.each { |cmd| system(cmd) }
        end
      end

      def self.display_cubes(list)
        table = Terminal::Table.new title: CUBES_TITLE, headings: CUBES_HEADERS do |t|
          [list].flatten.compact.each do |e|
            t.add_row(self.format_record(e))
          end
        end
        puts table
        puts "\n"
      end

      def self.format_record(record)
        cluster = self.format_cluster(record)
        container_status = record.respond_to?(:container_status) ? record.container_status : '-'
        [
          record.uuid,
          record.status,
          container_status,
          record.region,
          record.persistent_storage,
          record.ip_address || '-',
          record.port || '-',
          cluster
        ]
      end

      def self.format_cluster(record)
        return '-' unless o = record.cluster
        "#{o.type.singularize}:#{o.name}"
      end

      def self.display_logs(logs)
        puts logs
        puts "\n"
      end

      def self.display_snapshots(snapshots)
        table = Terminal::Table.new title: SNAPSHOTS_TITLE, headings: SNAPSHOTS_HEADERS do |t|
          [snapshots].flatten.compact.each do |e|
            t.add_row([
              e['created_at'],
              e['ip_address'],
              e['path']
            ])
          end
        end
        puts table
        puts "\n"
      end
    end
  end
end
